#include <esp_now.h>
#include <WiFi.h>
// Define pins for steering control
const int DIR1 = 26; //33; 
const int PWM1_PIN = 25;//32;
const int DIR2 = 33; 
const int PWM2_PIN = 32;
int button;
int avance_value = 0;
int direction_value = 0;

int test=0;
typedef struct joystick_data {
  int button;//première ligne du struct n'est pas lu est attribue directement la valeur 0
  int avance;
  int direction;
} joystick_data;

#define CHANNEL 1

// Init ESP Now with fallback
void InitESPNow() {
  WiFi.disconnect();
  if (esp_now_init() == ESP_OK) {
    Serial.println("ESPNow Init Success");
  }
  else {
    Serial.println("ESPNow Init Failed");
    // Retry InitESPNow, add a counte and then restart?
    // InitESPNow();
    // or Simply Restart
    ESP.restart();
  }
}

// config AP SSID
void configDeviceAP() {
  const char *SSID = "Slave_1";
  bool result = WiFi.softAP(SSID, "Slave_1_Password", CHANNEL, 0);
  if (!result) {
    Serial.println("AP Config failed.");
  } else {
    Serial.println("AP Config Success. Broadcasting with AP: " + String(SSID));
    Serial.print("AP CHANNEL "); Serial.println(WiFi.channel());
  }
}

void setup() {
  Serial.begin(115200);
  Serial.println("ESPNow/Basic/Slave Example");
  //Set device in AP mode to begin with
  WiFi.mode(WIFI_AP);
  // configure device AP mode
  configDeviceAP();
  // This is the mac address of the Slave in AP Mode
  Serial.print("AP MAC: "); Serial.println(WiFi.softAPmacAddress());
  // Init ESPNow with a fallback logic
  InitESPNow();
  // Once ESPNow is successfully Init, we will register for recv CB to
  // get recv packer info.
  esp_now_register_recv_cb(OnDataRecv);

  //Driver
  pinMode(DIR1, OUTPUT);//direction
  pinMode(PWM1_PIN, OUTPUT);//speed
  pinMode(DIR2, OUTPUT);//direction
  pinMode(PWM2_PIN, OUTPUT);//speed
  
}

// callback when data is recv from Master
void OnDataRecv(const uint8_t *mac_addr, const uint8_t *data, int data_len) {
  char macStr[18];
  snprintf(macStr, sizeof(macStr), "%02x:%02x:%02x:%02x:%02x:%02x",
           mac_addr[0], mac_addr[1], mac_addr[2], mac_addr[3], mac_addr[4], mac_addr[5]);
  Serial.print("Last Packet Recv from: "); Serial.println(macStr);
  joystick_data* test =(joystick_data*) data;
  Serial.print("Last Packet Recv Data: Avancement= "); Serial.print(test->avance);
  Serial.print(", Direction=");Serial.println(test->direction);
  //Serial.print(", ");Serial.println(test->button);
  
  avance_value = test->avance;
  direction_value = test->direction;
}

void loop() {
  // Map avance_value from                                                                                       0-4095 to 0-250 for PWM control
  int value_pwm = map(avance_value, 0, 4095, 0, 250);
  
  // Set both directions initially to forward
  digitalWrite(DIR1, HIGH);
  digitalWrite(DIR2, HIGH);

  // Determine how much to adjust PWM for each wheel based on direction
  int pwm_adjustment = 0;
  if (direction_value < 2600) { // Turn left
    pwm_adjustment = map(direction_value, 0, 2600, -value_pwm, 0); // Decrease right wheel speed
  } else if (direction_value > 2700) { // Turn right
    pwm_adjustment = map(direction_value, 2700, 4095, 0, value_pwm); // Decrease left wheel speed
  }
  
  // Apply PWM adjustments to each wheel
  int left_pwm = value_pwm + pwm_adjustment;
  int right_pwm = value_pwm - pwm_adjustment;
  
  // Ensure PWM values are within range
  left_pwm = constrain(left_pwm, 0, 250);
  right_pwm = constrain(right_pwm, 0, 250);


  // Apply PWM values to control the speed of each wheel
  analogWrite(PWM1_PIN, left_pwm);
  analogWrite(PWM2_PIN, right_pwm);

  delay(100);
}
